<div class="btn-group" role="group">
  <a href="{% url 'plugins:nautobot_deepcopy:copy' device.pk %}" class="btn btn-primary">
    <span class="glyphicon glyphicon-plus" aria-hidden="true"></span> Deep Copy</span>
  </a>
</div>
